/*
 * This file is part of gtkD.
 *
 * gtkD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version, with
 * some exceptions, please read the COPYING file.
 *
 * gtkD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with gtkD; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

// generated automatically - do not change
// find conversion definition on APILookup.txt
// implement new conversion functionalities on the wrap.utils pakage


module gobject.TypeInstance;

private import gobject.c.functions;
public  import gobject.c.types;
public  import gtkc.gobjecttypes;


/**
 * An opaque structure used as the base of all type instances.
 */
public class TypeInstance
{
	/** the main Gtk struct */
	protected GTypeInstance* gTypeInstance;
	protected bool ownedRef;

	/** Get the main Gtk struct */
	public GTypeInstance* getTypeInstanceStruct(bool transferOwnership = false)
	{
		if (transferOwnership)
			ownedRef = false;
		return gTypeInstance;
	}

	/** the main Gtk struct as a void* */
	protected void* getStruct()
	{
		return cast(void*)gTypeInstance;
	}

	/**
	 * Sets our main struct and passes it to the parent class.
	 */
	public this (GTypeInstance* gTypeInstance, bool ownedRef = false)
	{
		this.gTypeInstance = gTypeInstance;
		this.ownedRef = ownedRef;
	}


	/** */
	public void* getPrivate(GType privateType)
	{
		return g_type_instance_get_private(gTypeInstance, privateType);
	}
}
