/*
 * This file is part of gtkD.
 *
 * gtkD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version, with
 * some exceptions, please read the COPYING file.
 *
 * gtkD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with gtkD; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

// generated automatically - do not change
// find conversion definition on APILookup.txt
// implement new conversion functionalities on the wrap.utils pakage


module gdk.Keysyms;

/**
 * GdkKeysyms.
 */
public enum GdkKeysyms
{
	GDK_0 = 48,
	GDK_1 = 49,
	GDK_2 = 50,
	GDK_3 = 51,
	GDK_3270_AltCursor = 64784,
	GDK_3270_Attn = 64782,
	GDK_3270_BackTab = 64773,
	GDK_3270_ChangeScreen = 64793,
	GDK_3270_Copy = 64789,
	GDK_3270_CursorBlink = 64783,
	GDK_3270_CursorSelect = 64796,
	GDK_3270_DeleteWord = 64794,
	GDK_3270_Duplicate = 64769,
	GDK_3270_Enter = 64798,
	GDK_3270_EraseEOF = 64774,
	GDK_3270_EraseInput = 64775,
	GDK_3270_ExSelect = 64795,
	GDK_3270_FieldMark = 64770,
	GDK_3270_Ident = 64787,
	GDK_3270_Jump = 64786,
	GDK_3270_KeyClick = 64785,
	GDK_3270_Left2 = 64772,
	GDK_3270_PA1 = 64778,
	GDK_3270_PA2 = 64779,
	GDK_3270_PA3 = 64780,
	GDK_3270_Play = 64790,
	GDK_3270_PrintScreen = 64797,
	GDK_3270_Quit = 64777,
	GDK_3270_Record = 64792,
	GDK_3270_Reset = 64776,
	GDK_3270_Right2 = 64771,
	GDK_3270_Rule = 64788,
	GDK_3270_Setup = 64791,
	GDK_3270_Test = 64781,
	GDK_4 = 52,
	GDK_5 = 53,
	GDK_6 = 54,
	GDK_7 = 55,
	GDK_8 = 56,
	GDK_9 = 57,
	GDK_A = 65,
	GDK_AE = 198,
	GDK_Aacute = 193,
	GDK_Abelowdot = 16785056,
	GDK_Abreve = 451,
	GDK_Abreveacute = 16785070,
	GDK_Abrevebelowdot = 16785078,
	GDK_Abrevegrave = 16785072,
	GDK_Abrevehook = 16785074,
	GDK_Abrevetilde = 16785076,
	GDK_AccessX_Enable = 65136,
	GDK_AccessX_Feedback_Enable = 65137,
	GDK_Acircumflex = 194,
	GDK_Acircumflexacute = 16785060,
	GDK_Acircumflexbelowdot = 16785068,
	GDK_Acircumflexgrave = 16785062,
	GDK_Acircumflexhook = 16785064,
	GDK_Acircumflextilde = 16785066,
	GDK_AddFavorite = 269025081,
	GDK_Adiaeresis = 196,
	GDK_Agrave = 192,
	GDK_Ahook = 16785058,
	GDK_Alt_L = 65513,
	GDK_Alt_R = 65514,
	GDK_Amacron = 960,
	GDK_Aogonek = 417,
	GDK_ApplicationLeft = 269025104,
	GDK_ApplicationRight = 269025105,
	GDK_Arabic_0 = 16778848,
	GDK_Arabic_1 = 16778849,
	GDK_Arabic_2 = 16778850,
	GDK_Arabic_3 = 16778851,
	GDK_Arabic_4 = 16778852,
	GDK_Arabic_5 = 16778853,
	GDK_Arabic_6 = 16778854,
	GDK_Arabic_7 = 16778855,
	GDK_Arabic_8 = 16778856,
	GDK_Arabic_9 = 16778857,
	GDK_Arabic_ain = 1497,
	GDK_Arabic_alef = 1479,
	GDK_Arabic_alefmaksura = 1513,
	GDK_Arabic_beh = 1480,
	GDK_Arabic_comma = 1452,
	GDK_Arabic_dad = 1494,
	GDK_Arabic_dal = 1487,
	GDK_Arabic_damma = 1519,
	GDK_Arabic_dammatan = 1516,
	GDK_Arabic_ddal = 16778888,
	GDK_Arabic_farsi_yeh = 16778956,
	GDK_Arabic_fatha = 1518,
	GDK_Arabic_fathatan = 1515,
	GDK_Arabic_feh = 1505,
	GDK_Arabic_fullstop = 16778964,
	GDK_Arabic_gaf = 16778927,
	GDK_Arabic_ghain = 1498,
	GDK_Arabic_ha = 1511,
	GDK_Arabic_hah = 1485,
	GDK_Arabic_hamza = 1473,
	GDK_Arabic_hamza_above = 16778836,
	GDK_Arabic_hamza_below = 16778837,
	GDK_Arabic_hamzaonalef = 1475,
	GDK_Arabic_hamzaonwaw = 1476,
	GDK_Arabic_hamzaonyeh = 1478,
	GDK_Arabic_hamzaunderalef = 1477,
	GDK_Arabic_heh = 1511,
	GDK_Arabic_heh_doachashmee = 16778942,
	GDK_Arabic_heh_goal = 16778945,
	GDK_Arabic_jeem = 1484,
	GDK_Arabic_jeh = 16778904,
	GDK_Arabic_kaf = 1507,
	GDK_Arabic_kasra = 1520,
	GDK_Arabic_kasratan = 1517,
	GDK_Arabic_keheh = 16778921,
	GDK_Arabic_khah = 1486,
	GDK_Arabic_lam = 1508,
	GDK_Arabic_madda_above = 16778835,
	GDK_Arabic_maddaonalef = 1474,
	GDK_Arabic_meem = 1509,
	GDK_Arabic_noon = 1510,
	GDK_Arabic_noon_ghunna = 16778938,
	GDK_Arabic_peh = 16778878,
	GDK_Arabic_percent = 16778858,
	GDK_Arabic_qaf = 1506,
	GDK_Arabic_question_mark = 1471,
	GDK_Arabic_ra = 1489,
	GDK_Arabic_rreh = 16778897,
	GDK_Arabic_sad = 1493,
	GDK_Arabic_seen = 1491,
	GDK_Arabic_semicolon = 1467,
	GDK_Arabic_shadda = 1521,
	GDK_Arabic_sheen = 1492,
	GDK_Arabic_sukun = 1522,
	GDK_Arabic_superscript_alef = 16778864,
	GDK_Arabic_switch = 65406,
	GDK_Arabic_tah = 1495,
	GDK_Arabic_tatweel = 1504,
	GDK_Arabic_tcheh = 16778886,
	GDK_Arabic_teh = 1482,
	GDK_Arabic_tehmarbuta = 1481,
	GDK_Arabic_thal = 1488,
	GDK_Arabic_theh = 1483,
	GDK_Arabic_tteh = 16778873,
	GDK_Arabic_veh = 16778916,
	GDK_Arabic_waw = 1512,
	GDK_Arabic_yeh = 1514,
	GDK_Arabic_yeh_baree = 16778962,
	GDK_Arabic_zah = 1496,
	GDK_Arabic_zain = 1490,
	GDK_Aring = 197,
	GDK_Armenian_AT = 16778552,
	GDK_Armenian_AYB = 16778545,
	GDK_Armenian_BEN = 16778546,
	GDK_Armenian_CHA = 16778569,
	GDK_Armenian_DA = 16778548,
	GDK_Armenian_DZA = 16778561,
	GDK_Armenian_E = 16778551,
	GDK_Armenian_FE = 16778582,
	GDK_Armenian_GHAT = 16778562,
	GDK_Armenian_GIM = 16778547,
	GDK_Armenian_HI = 16778565,
	GDK_Armenian_HO = 16778560,
	GDK_Armenian_INI = 16778555,
	GDK_Armenian_JE = 16778571,
	GDK_Armenian_KE = 16778580,
	GDK_Armenian_KEN = 16778559,
	GDK_Armenian_KHE = 16778557,
	GDK_Armenian_LYUN = 16778556,
	GDK_Armenian_MEN = 16778564,
	GDK_Armenian_NU = 16778566,
	GDK_Armenian_O = 16778581,
	GDK_Armenian_PE = 16778570,
	GDK_Armenian_PYUR = 16778579,
	GDK_Armenian_RA = 16778572,
	GDK_Armenian_RE = 16778576,
	GDK_Armenian_SE = 16778573,
	GDK_Armenian_SHA = 16778567,
	GDK_Armenian_TCHE = 16778563,
	GDK_Armenian_TO = 16778553,
	GDK_Armenian_TSA = 16778558,
	GDK_Armenian_TSO = 16778577,
	GDK_Armenian_TYUN = 16778575,
	GDK_Armenian_VEV = 16778574,
	GDK_Armenian_VO = 16778568,
	GDK_Armenian_VYUN = 16778578,
	GDK_Armenian_YECH = 16778549,
	GDK_Armenian_ZA = 16778550,
	GDK_Armenian_ZHE = 16778554,
	GDK_Armenian_accent = 16778587,
	GDK_Armenian_amanak = 16778588,
	GDK_Armenian_apostrophe = 16778586,
	GDK_Armenian_at = 16778600,
	GDK_Armenian_ayb = 16778593,
	GDK_Armenian_ben = 16778594,
	GDK_Armenian_but = 16778589,
	GDK_Armenian_cha = 16778617,
	GDK_Armenian_da = 16778596,
	GDK_Armenian_dza = 16778609,
	GDK_Armenian_e = 16778599,
	GDK_Armenian_exclam = 16778588,
	GDK_Armenian_fe = 16778630,
	GDK_Armenian_full_stop = 16778633,
	GDK_Armenian_ghat = 16778610,
	GDK_Armenian_gim = 16778595,
	GDK_Armenian_hi = 16778613,
	GDK_Armenian_ho = 16778608,
	GDK_Armenian_hyphen = 16778634,
	GDK_Armenian_ini = 16778603,
	GDK_Armenian_je = 16778619,
	GDK_Armenian_ke = 16778628,
	GDK_Armenian_ken = 16778607,
	GDK_Armenian_khe = 16778605,
	GDK_Armenian_ligature_ew = 16778631,
	GDK_Armenian_lyun = 16778604,
	GDK_Armenian_men = 16778612,
	GDK_Armenian_nu = 16778614,
	GDK_Armenian_o = 16778629,
	GDK_Armenian_paruyk = 16778590,
	GDK_Armenian_pe = 16778618,
	GDK_Armenian_pyur = 16778627,
	GDK_Armenian_question = 16778590,
	GDK_Armenian_ra = 16778620,
	GDK_Armenian_re = 16778624,
	GDK_Armenian_se = 16778621,
	GDK_Armenian_separation_mark = 16778589,
	GDK_Armenian_sha = 16778615,
	GDK_Armenian_shesht = 16778587,
	GDK_Armenian_tche = 16778611,
	GDK_Armenian_to = 16778601,
	GDK_Armenian_tsa = 16778606,
	GDK_Armenian_tso = 16778625,
	GDK_Armenian_tyun = 16778623,
	GDK_Armenian_verjaket = 16778633,
	GDK_Armenian_vev = 16778622,
	GDK_Armenian_vo = 16778616,
	GDK_Armenian_vyun = 16778626,
	GDK_Armenian_yech = 16778597,
	GDK_Armenian_yentamna = 16778634,
	GDK_Armenian_za = 16778598,
	GDK_Armenian_zhe = 16778602,
	GDK_Atilde = 195,
	GDK_AudibleBell_Enable = 65146,
	GDK_AudioCycleTrack = 269025179,
	GDK_AudioForward = 269025175,
	GDK_AudioLowerVolume = 269025041,
	GDK_AudioMedia = 269025074,
	GDK_AudioMicMute = 269025202,
	GDK_AudioMute = 269025042,
	GDK_AudioNext = 269025047,
	GDK_AudioPause = 269025073,
	GDK_AudioPlay = 269025044,
	GDK_AudioPrev = 269025046,
	GDK_AudioRaiseVolume = 269025043,
	GDK_AudioRandomPlay = 269025177,
	GDK_AudioRecord = 269025052,
	GDK_AudioRepeat = 269025176,
	GDK_AudioRewind = 269025086,
	GDK_AudioStop = 269025045,
	GDK_Away = 269025165,
	GDK_B = 66,
	GDK_Babovedot = 16784898,
	GDK_Back = 269025062,
	GDK_BackForward = 269025087,
	GDK_BackSpace = 65288,
	GDK_Battery = 269025171,
	GDK_Begin = 65368,
	GDK_Blue = 269025190,
	GDK_Bluetooth = 269025172,
	GDK_Book = 269025106,
	GDK_BounceKeys_Enable = 65140,
	GDK_Break = 65387,
	GDK_BrightnessAdjust = 269025083,
	GDK_Byelorussian_SHORTU = 1726,
	GDK_Byelorussian_shortu = 1710,
	GDK_C = 67,
	GDK_CD = 269025107,
	GDK_CH = 65186,
	GDK_C_H = 65189,
	GDK_C_h = 65188,
	GDK_Cabovedot = 709,
	GDK_Cacute = 454,
	GDK_Calculator = 269025053,
	GDK_Calendar = 269025056,
	GDK_Cancel = 65385,
	GDK_Caps_Lock = 65509,
	GDK_Ccaron = 456,
	GDK_Ccedilla = 199,
	GDK_Ccircumflex = 710,
	GDK_Ch = 65185,
	GDK_Clear = 65291,
	GDK_ClearGrab = 269024801,
	GDK_Close = 269025110,
	GDK_Codeinput = 65335,
	GDK_ColonSign = 16785569,
	GDK_Community = 269025085,
	GDK_ContrastAdjust = 269025058,
	GDK_Control_L = 65507,
	GDK_Control_R = 65508,
	GDK_Copy = 269025111,
	GDK_CruzeiroSign = 16785570,
	GDK_Cut = 269025112,
	GDK_CycleAngle = 269025180,
	GDK_Cyrillic_A = 1761,
	GDK_Cyrillic_BE = 1762,
	GDK_Cyrillic_CHE = 1790,
	GDK_Cyrillic_CHE_descender = 16778422,
	GDK_Cyrillic_CHE_vertstroke = 16778424,
	GDK_Cyrillic_DE = 1764,
	GDK_Cyrillic_DZHE = 1727,
	GDK_Cyrillic_E = 1788,
	GDK_Cyrillic_EF = 1766,
	GDK_Cyrillic_EL = 1772,
	GDK_Cyrillic_EM = 1773,
	GDK_Cyrillic_EN = 1774,
	GDK_Cyrillic_EN_descender = 16778402,
	GDK_Cyrillic_ER = 1778,
	GDK_Cyrillic_ES = 1779,
	GDK_Cyrillic_GHE = 1767,
	GDK_Cyrillic_GHE_bar = 16778386,
	GDK_Cyrillic_HA = 1768,
	GDK_Cyrillic_HARDSIGN = 1791,
	GDK_Cyrillic_HA_descender = 16778418,
	GDK_Cyrillic_I = 1769,
	GDK_Cyrillic_IE = 1765,
	GDK_Cyrillic_IO = 1715,
	GDK_Cyrillic_I_macron = 16778466,
	GDK_Cyrillic_JE = 1720,
	GDK_Cyrillic_KA = 1771,
	GDK_Cyrillic_KA_descender = 16778394,
	GDK_Cyrillic_KA_vertstroke = 16778396,
	GDK_Cyrillic_LJE = 1721,
	GDK_Cyrillic_NJE = 1722,
	GDK_Cyrillic_O = 1775,
	GDK_Cyrillic_O_bar = 16778472,
	GDK_Cyrillic_PE = 1776,
	GDK_Cyrillic_SCHWA = 16778456,
	GDK_Cyrillic_SHA = 1787,
	GDK_Cyrillic_SHCHA = 1789,
	GDK_Cyrillic_SHHA = 16778426,
	GDK_Cyrillic_SHORTI = 1770,
	GDK_Cyrillic_SOFTSIGN = 1784,
	GDK_Cyrillic_TE = 1780,
	GDK_Cyrillic_TSE = 1763,
	GDK_Cyrillic_U = 1781,
	GDK_Cyrillic_U_macron = 16778478,
	GDK_Cyrillic_U_straight = 16778414,
	GDK_Cyrillic_U_straight_bar = 16778416,
	GDK_Cyrillic_VE = 1783,
	GDK_Cyrillic_YA = 1777,
	GDK_Cyrillic_YERU = 1785,
	GDK_Cyrillic_YU = 1760,
	GDK_Cyrillic_ZE = 1786,
	GDK_Cyrillic_ZHE = 1782,
	GDK_Cyrillic_ZHE_descender = 16778390,
	GDK_Cyrillic_a = 1729,
	GDK_Cyrillic_be = 1730,
	GDK_Cyrillic_che = 1758,
	GDK_Cyrillic_che_descender = 16778423,
	GDK_Cyrillic_che_vertstroke = 16778425,
	GDK_Cyrillic_de = 1732,
	GDK_Cyrillic_dzhe = 1711,
	GDK_Cyrillic_e = 1756,
	GDK_Cyrillic_ef = 1734,
	GDK_Cyrillic_el = 1740,
	GDK_Cyrillic_em = 1741,
	GDK_Cyrillic_en = 1742,
	GDK_Cyrillic_en_descender = 16778403,
	GDK_Cyrillic_er = 1746,
	GDK_Cyrillic_es = 1747,
	GDK_Cyrillic_ghe = 1735,
	GDK_Cyrillic_ghe_bar = 16778387,
	GDK_Cyrillic_ha = 1736,
	GDK_Cyrillic_ha_descender = 16778419,
	GDK_Cyrillic_hardsign = 1759,
	GDK_Cyrillic_i = 1737,
	GDK_Cyrillic_i_macron = 16778467,
	GDK_Cyrillic_ie = 1733,
	GDK_Cyrillic_io = 1699,
	GDK_Cyrillic_je = 1704,
	GDK_Cyrillic_ka = 1739,
	GDK_Cyrillic_ka_descender = 16778395,
	GDK_Cyrillic_ka_vertstroke = 16778397,
	GDK_Cyrillic_lje = 1705,
	GDK_Cyrillic_nje = 1706,
	GDK_Cyrillic_o = 1743,
	GDK_Cyrillic_o_bar = 16778473,
	GDK_Cyrillic_pe = 1744,
	GDK_Cyrillic_schwa = 16778457,
	GDK_Cyrillic_sha = 1755,
	GDK_Cyrillic_shcha = 1757,
	GDK_Cyrillic_shha = 16778427,
	GDK_Cyrillic_shorti = 1738,
	GDK_Cyrillic_softsign = 1752,
	GDK_Cyrillic_te = 1748,
	GDK_Cyrillic_tse = 1731,
	GDK_Cyrillic_u = 1749,
	GDK_Cyrillic_u_macron = 16778479,
	GDK_Cyrillic_u_straight = 16778415,
	GDK_Cyrillic_u_straight_bar = 16778417,
	GDK_Cyrillic_ve = 1751,
	GDK_Cyrillic_ya = 1745,
	GDK_Cyrillic_yeru = 1753,
	GDK_Cyrillic_yu = 1728,
	GDK_Cyrillic_ze = 1754,
	GDK_Cyrillic_zhe = 1750,
	GDK_Cyrillic_zhe_descender = 16778391,
	GDK_D = 68,
	GDK_DOS = 269025114,
	GDK_Dabovedot = 16784906,
	GDK_Dcaron = 463,
	GDK_Delete = 65535,
	GDK_Display = 269025113,
	GDK_Documents = 269025115,
	GDK_DongSign = 16785579,
	GDK_Down = 65364,
	GDK_Dstroke = 464,
	GDK_E = 69,
	GDK_ENG = 957,
	GDK_ETH = 208,
	GDK_EZH = 16777655,
	GDK_Eabovedot = 972,
	GDK_Eacute = 201,
	GDK_Ebelowdot = 16785080,
	GDK_Ecaron = 460,
	GDK_Ecircumflex = 202,
	GDK_Ecircumflexacute = 16785086,
	GDK_Ecircumflexbelowdot = 16785094,
	GDK_Ecircumflexgrave = 16785088,
	GDK_Ecircumflexhook = 16785090,
	GDK_Ecircumflextilde = 16785092,
	GDK_EcuSign = 16785568,
	GDK_Ediaeresis = 203,
	GDK_Egrave = 200,
	GDK_Ehook = 16785082,
	GDK_Eisu_Shift = 65327,
	GDK_Eisu_toggle = 65328,
	GDK_Eject = 269025068,
	GDK_Emacron = 938,
	GDK_End = 65367,
	GDK_Eogonek = 458,
	GDK_Escape = 65307,
	GDK_Eth = 208,
	GDK_Etilde = 16785084,
	GDK_EuroSign = 8364,
	GDK_Excel = 269025116,
	GDK_Execute = 65378,
	GDK_Explorer = 269025117,
	GDK_F = 70,
	GDK_F1 = 65470,
	GDK_F10 = 65479,
	GDK_F11 = 65480,
	GDK_F12 = 65481,
	GDK_F13 = 65482,
	GDK_F14 = 65483,
	GDK_F15 = 65484,
	GDK_F16 = 65485,
	GDK_F17 = 65486,
	GDK_F18 = 65487,
	GDK_F19 = 65488,
	GDK_F2 = 65471,
	GDK_F20 = 65489,
	GDK_F21 = 65490,
	GDK_F22 = 65491,
	GDK_F23 = 65492,
	GDK_F24 = 65493,
	GDK_F25 = 65494,
	GDK_F26 = 65495,
	GDK_F27 = 65496,
	GDK_F28 = 65497,
	GDK_F29 = 65498,
	GDK_F3 = 65472,
	GDK_F30 = 65499,
	GDK_F31 = 65500,
	GDK_F32 = 65501,
	GDK_F33 = 65502,
	GDK_F34 = 65503,
	GDK_F35 = 65504,
	GDK_F4 = 65473,
	GDK_F5 = 65474,
	GDK_F6 = 65475,
	GDK_F7 = 65476,
	GDK_F8 = 65477,
	GDK_F9 = 65478,
	GDK_FFrancSign = 16785571,
	GDK_Fabovedot = 16784926,
	GDK_Farsi_0 = 16778992,
	GDK_Farsi_1 = 16778993,
	GDK_Farsi_2 = 16778994,
	GDK_Farsi_3 = 16778995,
	GDK_Farsi_4 = 16778996,
	GDK_Farsi_5 = 16778997,
	GDK_Farsi_6 = 16778998,
	GDK_Farsi_7 = 16778999,
	GDK_Farsi_8 = 16779000,
	GDK_Farsi_9 = 16779001,
	GDK_Farsi_yeh = 16778956,
	GDK_Favorites = 269025072,
	GDK_Finance = 269025084,
	GDK_Find = 65384,
	GDK_First_Virtual_Screen = 65232,
	GDK_Forward = 269025063,
	GDK_FrameBack = 269025181,
	GDK_FrameForward = 269025182,
	GDK_G = 71,
	GDK_Gabovedot = 725,
	GDK_Game = 269025118,
	GDK_Gbreve = 683,
	GDK_Gcaron = 16777702,
	GDK_Gcedilla = 939,
	GDK_Gcircumflex = 728,
	GDK_Georgian_an = 16781520,
	GDK_Georgian_ban = 16781521,
	GDK_Georgian_can = 16781546,
	GDK_Georgian_char = 16781549,
	GDK_Georgian_chin = 16781545,
	GDK_Georgian_cil = 16781548,
	GDK_Georgian_don = 16781523,
	GDK_Georgian_en = 16781524,
	GDK_Georgian_fi = 16781558,
	GDK_Georgian_gan = 16781522,
	GDK_Georgian_ghan = 16781542,
	GDK_Georgian_hae = 16781552,
	GDK_Georgian_har = 16781556,
	GDK_Georgian_he = 16781553,
	GDK_Georgian_hie = 16781554,
	GDK_Georgian_hoe = 16781557,
	GDK_Georgian_in = 16781528,
	GDK_Georgian_jhan = 16781551,
	GDK_Georgian_jil = 16781547,
	GDK_Georgian_kan = 16781529,
	GDK_Georgian_khar = 16781541,
	GDK_Georgian_las = 16781530,
	GDK_Georgian_man = 16781531,
	GDK_Georgian_nar = 16781532,
	GDK_Georgian_on = 16781533,
	GDK_Georgian_par = 16781534,
	GDK_Georgian_phar = 16781540,
	GDK_Georgian_qar = 16781543,
	GDK_Georgian_rae = 16781536,
	GDK_Georgian_san = 16781537,
	GDK_Georgian_shin = 16781544,
	GDK_Georgian_tan = 16781527,
	GDK_Georgian_tar = 16781538,
	GDK_Georgian_un = 16781539,
	GDK_Georgian_vin = 16781525,
	GDK_Georgian_we = 16781555,
	GDK_Georgian_xan = 16781550,
	GDK_Georgian_zen = 16781526,
	GDK_Georgian_zhar = 16781535,
	GDK_Go = 269025119,
	GDK_Greek_ALPHA = 1985,
	GDK_Greek_ALPHAaccent = 1953,
	GDK_Greek_BETA = 1986,
	GDK_Greek_CHI = 2007,
	GDK_Greek_DELTA = 1988,
	GDK_Greek_EPSILON = 1989,
	GDK_Greek_EPSILONaccent = 1954,
	GDK_Greek_ETA = 1991,
	GDK_Greek_ETAaccent = 1955,
	GDK_Greek_GAMMA = 1987,
	GDK_Greek_IOTA = 1993,
	GDK_Greek_IOTAaccent = 1956,
	GDK_Greek_IOTAdiaeresis = 1957,
	GDK_Greek_IOTAdieresis = 1957,
	GDK_Greek_KAPPA = 1994,
	GDK_Greek_LAMBDA = 1995,
	GDK_Greek_LAMDA = 1995,
	GDK_Greek_MU = 1996,
	GDK_Greek_NU = 1997,
	GDK_Greek_OMEGA = 2009,
	GDK_Greek_OMEGAaccent = 1963,
	GDK_Greek_OMICRON = 1999,
	GDK_Greek_OMICRONaccent = 1959,
	GDK_Greek_PHI = 2006,
	GDK_Greek_PI = 2000,
	GDK_Greek_PSI = 2008,
	GDK_Greek_RHO = 2001,
	GDK_Greek_SIGMA = 2002,
	GDK_Greek_TAU = 2004,
	GDK_Greek_THETA = 1992,
	GDK_Greek_UPSILON = 2005,
	GDK_Greek_UPSILONaccent = 1960,
	GDK_Greek_UPSILONdieresis = 1961,
	GDK_Greek_XI = 1998,
	GDK_Greek_ZETA = 1990,
	GDK_Greek_accentdieresis = 1966,
	GDK_Greek_alpha = 2017,
	GDK_Greek_alphaaccent = 1969,
	GDK_Greek_beta = 2018,
	GDK_Greek_chi = 2039,
	GDK_Greek_delta = 2020,
	GDK_Greek_epsilon = 2021,
	GDK_Greek_epsilonaccent = 1970,
	GDK_Greek_eta = 2023,
	GDK_Greek_etaaccent = 1971,
	GDK_Greek_finalsmallsigma = 2035,
	GDK_Greek_gamma = 2019,
	GDK_Greek_horizbar = 1967,
	GDK_Greek_iota = 2025,
	GDK_Greek_iotaaccent = 1972,
	GDK_Greek_iotaaccentdieresis = 1974,
	GDK_Greek_iotadieresis = 1973,
	GDK_Greek_kappa = 2026,
	GDK_Greek_lambda = 2027,
	GDK_Greek_lamda = 2027,
	GDK_Greek_mu = 2028,
	GDK_Greek_nu = 2029,
	GDK_Greek_omega = 2041,
	GDK_Greek_omegaaccent = 1979,
	GDK_Greek_omicron = 2031,
	GDK_Greek_omicronaccent = 1975,
	GDK_Greek_phi = 2038,
	GDK_Greek_pi = 2032,
	GDK_Greek_psi = 2040,
	GDK_Greek_rho = 2033,
	GDK_Greek_sigma = 2034,
	GDK_Greek_switch = 65406,
	GDK_Greek_tau = 2036,
	GDK_Greek_theta = 2024,
	GDK_Greek_upsilon = 2037,
	GDK_Greek_upsilonaccent = 1976,
	GDK_Greek_upsilonaccentdieresis = 1978,
	GDK_Greek_upsilondieresis = 1977,
	GDK_Greek_xi = 2030,
	GDK_Greek_zeta = 2022,
	GDK_Green = 269025188,
	GDK_H = 72,
	GDK_Hangul = 65329,
	GDK_Hangul_A = 3775,
	GDK_Hangul_AE = 3776,
	GDK_Hangul_AraeA = 3830,
	GDK_Hangul_AraeAE = 3831,
	GDK_Hangul_Banja = 65337,
	GDK_Hangul_Cieuc = 3770,
	GDK_Hangul_Codeinput = 65335,
	GDK_Hangul_Dikeud = 3751,
	GDK_Hangul_E = 3780,
	GDK_Hangul_EO = 3779,
	GDK_Hangul_EU = 3793,
	GDK_Hangul_End = 65331,
	GDK_Hangul_Hanja = 65332,
	GDK_Hangul_Hieuh = 3774,
	GDK_Hangul_I = 3795,
	GDK_Hangul_Ieung = 3767,
	GDK_Hangul_J_Cieuc = 3818,
	GDK_Hangul_J_Dikeud = 3802,
	GDK_Hangul_J_Hieuh = 3822,
	GDK_Hangul_J_Ieung = 3816,
	GDK_Hangul_J_Jieuj = 3817,
	GDK_Hangul_J_Khieuq = 3819,
	GDK_Hangul_J_Kiyeog = 3796,
	GDK_Hangul_J_KiyeogSios = 3798,
	GDK_Hangul_J_KkogjiDalrinIeung = 3833,
	GDK_Hangul_J_Mieum = 3811,
	GDK_Hangul_J_Nieun = 3799,
	GDK_Hangul_J_NieunHieuh = 3801,
	GDK_Hangul_J_NieunJieuj = 3800,
	GDK_Hangul_J_PanSios = 3832,
	GDK_Hangul_J_Phieuf = 3821,
	GDK_Hangul_J_Pieub = 3812,
	GDK_Hangul_J_PieubSios = 3813,
	GDK_Hangul_J_Rieul = 3803,
	GDK_Hangul_J_RieulHieuh = 3810,
	GDK_Hangul_J_RieulKiyeog = 3804,
	GDK_Hangul_J_RieulMieum = 3805,
	GDK_Hangul_J_RieulPhieuf = 3809,
	GDK_Hangul_J_RieulPieub = 3806,
	GDK_Hangul_J_RieulSios = 3807,
	GDK_Hangul_J_RieulTieut = 3808,
	GDK_Hangul_J_Sios = 3814,
	GDK_Hangul_J_SsangKiyeog = 3797,
	GDK_Hangul_J_SsangSios = 3815,
	GDK_Hangul_J_Tieut = 3820,
	GDK_Hangul_J_YeorinHieuh = 3834,
	GDK_Hangul_Jamo = 65333,
	GDK_Hangul_Jeonja = 65336,
	GDK_Hangul_Jieuj = 3768,
	GDK_Hangul_Khieuq = 3771,
	GDK_Hangul_Kiyeog = 3745,
	GDK_Hangul_KiyeogSios = 3747,
	GDK_Hangul_KkogjiDalrinIeung = 3827,
	GDK_Hangul_Mieum = 3761,
	GDK_Hangul_MultipleCandidate = 65341,
	GDK_Hangul_Nieun = 3748,
	GDK_Hangul_NieunHieuh = 3750,
	GDK_Hangul_NieunJieuj = 3749,
	GDK_Hangul_O = 3783,
	GDK_Hangul_OE = 3786,
	GDK_Hangul_PanSios = 3826,
	GDK_Hangul_Phieuf = 3773,
	GDK_Hangul_Pieub = 3762,
	GDK_Hangul_PieubSios = 3764,
	GDK_Hangul_PostHanja = 65339,
	GDK_Hangul_PreHanja = 65338,
	GDK_Hangul_PreviousCandidate = 65342,
	GDK_Hangul_Rieul = 3753,
	GDK_Hangul_RieulHieuh = 3760,
	GDK_Hangul_RieulKiyeog = 3754,
	GDK_Hangul_RieulMieum = 3755,
	GDK_Hangul_RieulPhieuf = 3759,
	GDK_Hangul_RieulPieub = 3756,
	GDK_Hangul_RieulSios = 3757,
	GDK_Hangul_RieulTieut = 3758,
	GDK_Hangul_RieulYeorinHieuh = 3823,
	GDK_Hangul_Romaja = 65334,
	GDK_Hangul_SingleCandidate = 65340,
	GDK_Hangul_Sios = 3765,
	GDK_Hangul_Special = 65343,
	GDK_Hangul_SsangDikeud = 3752,
	GDK_Hangul_SsangJieuj = 3769,
	GDK_Hangul_SsangKiyeog = 3746,
	GDK_Hangul_SsangPieub = 3763,
	GDK_Hangul_SsangSios = 3766,
	GDK_Hangul_Start = 65330,
	GDK_Hangul_SunkyeongeumMieum = 3824,
	GDK_Hangul_SunkyeongeumPhieuf = 3828,
	GDK_Hangul_SunkyeongeumPieub = 3825,
	GDK_Hangul_Tieut = 3772,
	GDK_Hangul_U = 3788,
	GDK_Hangul_WA = 3784,
	GDK_Hangul_WAE = 3785,
	GDK_Hangul_WE = 3790,
	GDK_Hangul_WEO = 3789,
	GDK_Hangul_WI = 3791,
	GDK_Hangul_YA = 3777,
	GDK_Hangul_YAE = 3778,
	GDK_Hangul_YE = 3782,
	GDK_Hangul_YEO = 3781,
	GDK_Hangul_YI = 3794,
	GDK_Hangul_YO = 3787,
	GDK_Hangul_YU = 3792,
	GDK_Hangul_YeorinHieuh = 3829,
	GDK_Hangul_switch = 65406,
	GDK_Hankaku = 65321,
	GDK_Hcircumflex = 678,
	GDK_Hebrew_switch = 65406,
	GDK_Help = 65386,
	GDK_Henkan = 65315,
	GDK_Henkan_Mode = 65315,
	GDK_Hibernate = 269025192,
	GDK_Hiragana = 65317,
	GDK_Hiragana_Katakana = 65319,
	GDK_History = 269025079,
	GDK_Home = 65360,
	GDK_HomePage = 269025048,
	GDK_HotLinks = 269025082,
	GDK_Hstroke = 673,
	GDK_Hyper_L = 65517,
	GDK_Hyper_R = 65518,
	GDK_I = 73,
	GDK_ISO_Center_Object = 65075,
	GDK_ISO_Continuous_Underline = 65072,
	GDK_ISO_Discontinuous_Underline = 65073,
	GDK_ISO_Emphasize = 65074,
	GDK_ISO_Enter = 65076,
	GDK_ISO_Fast_Cursor_Down = 65071,
	GDK_ISO_Fast_Cursor_Left = 65068,
	GDK_ISO_Fast_Cursor_Right = 65069,
	GDK_ISO_Fast_Cursor_Up = 65070,
	GDK_ISO_First_Group = 65036,
	GDK_ISO_First_Group_Lock = 65037,
	GDK_ISO_Group_Latch = 65030,
	GDK_ISO_Group_Lock = 65031,
	GDK_ISO_Group_Shift = 65406,
	GDK_ISO_Last_Group = 65038,
	GDK_ISO_Last_Group_Lock = 65039,
	GDK_ISO_Left_Tab = 65056,
	GDK_ISO_Level2_Latch = 65026,
	GDK_ISO_Level3_Latch = 65028,
	GDK_ISO_Level3_Lock = 65029,
	GDK_ISO_Level3_Shift = 65027,
	GDK_ISO_Level5_Latch = 65042,
	GDK_ISO_Level5_Lock = 65043,
	GDK_ISO_Level5_Shift = 65041,
	GDK_ISO_Lock = 65025,
	GDK_ISO_Move_Line_Down = 65058,
	GDK_ISO_Move_Line_Up = 65057,
	GDK_ISO_Next_Group = 65032,
	GDK_ISO_Next_Group_Lock = 65033,
	GDK_ISO_Partial_Line_Down = 65060,
	GDK_ISO_Partial_Line_Up = 65059,
	GDK_ISO_Partial_Space_Left = 65061,
	GDK_ISO_Partial_Space_Right = 65062,
	GDK_ISO_Prev_Group = 65034,
	GDK_ISO_Prev_Group_Lock = 65035,
	GDK_ISO_Release_Both_Margins = 65067,
	GDK_ISO_Release_Margin_Left = 65065,
	GDK_ISO_Release_Margin_Right = 65066,
	GDK_ISO_Set_Margin_Left = 65063,
	GDK_ISO_Set_Margin_Right = 65064,
	GDK_Iabovedot = 681,
	GDK_Iacute = 205,
	GDK_Ibelowdot = 16785098,
	GDK_Ibreve = 16777516,
	GDK_Icircumflex = 206,
	GDK_Idiaeresis = 207,
	GDK_Igrave = 204,
	GDK_Ihook = 16785096,
	GDK_Imacron = 975,
	GDK_Insert = 65379,
	GDK_Iogonek = 967,
	GDK_Itilde = 933,
	GDK_J = 74,
	GDK_Jcircumflex = 684,
	GDK_K = 75,
	GDK_KP_0 = 65456,
	GDK_KP_1 = 65457,
	GDK_KP_2 = 65458,
	GDK_KP_3 = 65459,
	GDK_KP_4 = 65460,
	GDK_KP_5 = 65461,
	GDK_KP_6 = 65462,
	GDK_KP_7 = 65463,
	GDK_KP_8 = 65464,
	GDK_KP_9 = 65465,
	GDK_KP_Add = 65451,
	GDK_KP_Begin = 65437,
	GDK_KP_Decimal = 65454,
	GDK_KP_Delete = 65439,
	GDK_KP_Divide = 65455,
	GDK_KP_Down = 65433,
	GDK_KP_End = 65436,
	GDK_KP_Enter = 65421,
	GDK_KP_Equal = 65469,
	GDK_KP_F1 = 65425,
	GDK_KP_F2 = 65426,
	GDK_KP_F3 = 65427,
	GDK_KP_F4 = 65428,
	GDK_KP_Home = 65429,
	GDK_KP_Insert = 65438,
	GDK_KP_Left = 65430,
	GDK_KP_Multiply = 65450,
	GDK_KP_Next = 65435,
	GDK_KP_Page_Down = 65435,
	GDK_KP_Page_Up = 65434,
	GDK_KP_Prior = 65434,
	GDK_KP_Right = 65432,
	GDK_KP_Separator = 65452,
	GDK_KP_Space = 65408,
	GDK_KP_Subtract = 65453,
	GDK_KP_Tab = 65417,
	GDK_KP_Up = 65431,
	GDK_Kana_Lock = 65325,
	GDK_Kana_Shift = 65326,
	GDK_Kanji = 65313,
	GDK_Kanji_Bangou = 65335,
	GDK_Katakana = 65318,
	GDK_KbdBrightnessDown = 269025030,
	GDK_KbdBrightnessUp = 269025029,
	GDK_KbdLightOnOff = 269025028,
	GDK_Kcedilla = 979,
	GDK_Korean_Won = 3839,
	GDK_L = 76,
	GDK_L1 = 65480,
	GDK_L10 = 65489,
	GDK_L2 = 65481,
	GDK_L3 = 65482,
	GDK_L4 = 65483,
	GDK_L5 = 65484,
	GDK_L6 = 65485,
	GDK_L7 = 65486,
	GDK_L8 = 65487,
	GDK_L9 = 65488,
	GDK_Lacute = 453,
	GDK_Last_Virtual_Screen = 65236,
	GDK_Launch0 = 269025088,
	GDK_Launch1 = 269025089,
	GDK_Launch2 = 269025090,
	GDK_Launch3 = 269025091,
	GDK_Launch4 = 269025092,
	GDK_Launch5 = 269025093,
	GDK_Launch6 = 269025094,
	GDK_Launch7 = 269025095,
	GDK_Launch8 = 269025096,
	GDK_Launch9 = 269025097,
	GDK_LaunchA = 269025098,
	GDK_LaunchB = 269025099,
	GDK_LaunchC = 269025100,
	GDK_LaunchD = 269025101,
	GDK_LaunchE = 269025102,
	GDK_LaunchF = 269025103,
	GDK_Lbelowdot = 16784950,
	GDK_Lcaron = 421,
	GDK_Lcedilla = 934,
	GDK_Left = 65361,
	GDK_LightBulb = 269025077,
	GDK_Linefeed = 65290,
	GDK_LiraSign = 16785572,
	GDK_LogGrabInfo = 269024805,
	GDK_LogOff = 269025121,
	GDK_LogWindowTree = 269024804,
	GDK_Lstroke = 419,
	GDK_M = 77,
	GDK_Mabovedot = 16784960,
	GDK_Macedonia_DSE = 1717,
	GDK_Macedonia_GJE = 1714,
	GDK_Macedonia_KJE = 1724,
	GDK_Macedonia_dse = 1701,
	GDK_Macedonia_gje = 1698,
	GDK_Macedonia_kje = 1708,
	GDK_Mae_Koho = 65342,
	GDK_Mail = 269025049,
	GDK_MailForward = 269025168,
	GDK_Market = 269025122,
	GDK_Massyo = 65324,
	GDK_Meeting = 269025123,
	GDK_Memo = 269025054,
	GDK_Menu = 65383,
	GDK_MenuKB = 269025125,
	GDK_MenuPB = 269025126,
	GDK_Messenger = 269025166,
	GDK_Meta_L = 65511,
	GDK_Meta_R = 65512,
	GDK_MillSign = 16785573,
	GDK_ModeLock = 269025025,
	GDK_Mode_switch = 65406,
	GDK_MonBrightnessDown = 269025027,
	GDK_MonBrightnessUp = 269025026,
	GDK_MouseKeys_Accel_Enable = 65143,
	GDK_MouseKeys_Enable = 65142,
	GDK_Muhenkan = 65314,
	GDK_Multi_key = 65312,
	GDK_MultipleCandidate = 65341,
	GDK_Music = 269025170,
	GDK_MyComputer = 269025075,
	GDK_MySites = 269025127,
	GDK_N = 78,
	GDK_Nacute = 465,
	GDK_NairaSign = 16785574,
	GDK_Ncaron = 466,
	GDK_Ncedilla = 977,
	GDK_New = 269025128,
	GDK_NewSheqelSign = 16785578,
	GDK_News = 269025129,
	GDK_Next = 65366,
	GDK_Next_VMode = 269024802,
	GDK_Next_Virtual_Screen = 65234,
	GDK_Ntilde = 209,
	GDK_Num_Lock = 65407,
	GDK_O = 79,
	GDK_OE = 5052,
	GDK_Oacute = 211,
	GDK_Obarred = 16777631,
	GDK_Obelowdot = 16785100,
	GDK_Ocaron = 16777681,
	GDK_Ocircumflex = 212,
	GDK_Ocircumflexacute = 16785104,
	GDK_Ocircumflexbelowdot = 16785112,
	GDK_Ocircumflexgrave = 16785106,
	GDK_Ocircumflexhook = 16785108,
	GDK_Ocircumflextilde = 16785110,
	GDK_Odiaeresis = 214,
	GDK_Odoubleacute = 469,
	GDK_OfficeHome = 269025130,
	GDK_Ograve = 210,
	GDK_Ohook = 16785102,
	GDK_Ohorn = 16777632,
	GDK_Ohornacute = 16785114,
	GDK_Ohornbelowdot = 16785122,
	GDK_Ohorngrave = 16785116,
	GDK_Ohornhook = 16785118,
	GDK_Ohorntilde = 16785120,
	GDK_Omacron = 978,
	GDK_Ooblique = 216,
	GDK_Open = 269025131,
	GDK_OpenURL = 269025080,
	GDK_Option = 269025132,
	GDK_Oslash = 216,
	GDK_Otilde = 213,
	GDK_Overlay1_Enable = 65144,
	GDK_Overlay2_Enable = 65145,
	GDK_P = 80,
	GDK_Pabovedot = 16784982,
	GDK_Page_Down = 65366,
	GDK_Page_Up = 65365,
	GDK_Paste = 269025133,
	GDK_Pause = 65299,
	GDK_PesetaSign = 16785575,
	GDK_Phone = 269025134,
	GDK_Pictures = 269025169,
	GDK_Pointer_Accelerate = 65274,
	GDK_Pointer_Button1 = 65257,
	GDK_Pointer_Button2 = 65258,
	GDK_Pointer_Button3 = 65259,
	GDK_Pointer_Button4 = 65260,
	GDK_Pointer_Button5 = 65261,
	GDK_Pointer_Button_Dflt = 65256,
	GDK_Pointer_DblClick1 = 65263,
	GDK_Pointer_DblClick2 = 65264,
	GDK_Pointer_DblClick3 = 65265,
	GDK_Pointer_DblClick4 = 65266,
	GDK_Pointer_DblClick5 = 65267,
	GDK_Pointer_DblClick_Dflt = 65262,
	GDK_Pointer_DfltBtnNext = 65275,
	GDK_Pointer_DfltBtnPrev = 65276,
	GDK_Pointer_Down = 65251,
	GDK_Pointer_DownLeft = 65254,
	GDK_Pointer_DownRight = 65255,
	GDK_Pointer_Drag1 = 65269,
	GDK_Pointer_Drag2 = 65270,
	GDK_Pointer_Drag3 = 65271,
	GDK_Pointer_Drag4 = 65272,
	GDK_Pointer_Drag5 = 65277,
	GDK_Pointer_Drag_Dflt = 65268,
	GDK_Pointer_EnableKeys = 65273,
	GDK_Pointer_Left = 65248,
	GDK_Pointer_Right = 65249,
	GDK_Pointer_Up = 65250,
	GDK_Pointer_UpLeft = 65252,
	GDK_Pointer_UpRight = 65253,
	GDK_PowerDown = 269025057,
	GDK_PowerOff = 269025066,
	GDK_Prev_VMode = 269024803,
	GDK_Prev_Virtual_Screen = 65233,
	GDK_PreviousCandidate = 65342,
	GDK_Print = 65377,
	GDK_Prior = 65365,
	GDK_Q = 81,
	GDK_R = 82,
	GDK_R1 = 65490,
	GDK_R10 = 65499,
	GDK_R11 = 65500,
	GDK_R12 = 65501,
	GDK_R13 = 65502,
	GDK_R14 = 65503,
	GDK_R15 = 65504,
	GDK_R2 = 65491,
	GDK_R3 = 65492,
	GDK_R4 = 65493,
	GDK_R5 = 65494,
	GDK_R6 = 65495,
	GDK_R7 = 65496,
	GDK_R8 = 65497,
	GDK_R9 = 65498,
	GDK_Racute = 448,
	GDK_Rcaron = 472,
	GDK_Rcedilla = 931,
	GDK_Red = 269025187,
	GDK_Redo = 65382,
	GDK_Refresh = 269025065,
	GDK_Reload = 269025139,
	GDK_RepeatKeys_Enable = 65138,
	GDK_Reply = 269025138,
	GDK_Return = 65293,
	GDK_Right = 65363,
	GDK_RockerDown = 269025060,
	GDK_RockerEnter = 269025061,
	GDK_RockerUp = 269025059,
	GDK_Romaji = 65316,
	GDK_RotateWindows = 269025140,
	GDK_RotationKB = 269025142,
	GDK_RotationPB = 269025141,
	GDK_RupeeSign = 16785576,
	GDK_S = 83,
	GDK_SCHWA = 16777615,
	GDK_Sabovedot = 16784992,
	GDK_Sacute = 422,
	GDK_Save = 269025143,
	GDK_Scaron = 425,
	GDK_Scedilla = 426,
	GDK_Scircumflex = 734,
	GDK_ScreenSaver = 269025069,
	GDK_ScrollClick = 269025146,
	GDK_ScrollDown = 269025145,
	GDK_ScrollUp = 269025144,
	GDK_Scroll_Lock = 65300,
	GDK_Search = 269025051,
	GDK_Select = 65376,
	GDK_SelectButton = 269025184,
	GDK_Send = 269025147,
	GDK_Serbian_DJE = 1713,
	GDK_Serbian_DZE = 1727,
	GDK_Serbian_JE = 1720,
	GDK_Serbian_LJE = 1721,
	GDK_Serbian_NJE = 1722,
	GDK_Serbian_TSHE = 1723,
	GDK_Serbian_dje = 1697,
	GDK_Serbian_dze = 1711,
	GDK_Serbian_je = 1704,
	GDK_Serbian_lje = 1705,
	GDK_Serbian_nje = 1706,
	GDK_Serbian_tshe = 1707,
	GDK_Shift_L = 65505,
	GDK_Shift_Lock = 65510,
	GDK_Shift_R = 65506,
	GDK_Shop = 269025078,
	GDK_SingleCandidate = 65340,
	GDK_Sinh_a = 16780677,
	GDK_Sinh_aa = 16780678,
	GDK_Sinh_aa2 = 16780751,
	GDK_Sinh_ae = 16780679,
	GDK_Sinh_ae2 = 16780752,
	GDK_Sinh_aee = 16780680,
	GDK_Sinh_aee2 = 16780753,
	GDK_Sinh_ai = 16780691,
	GDK_Sinh_ai2 = 16780763,
	GDK_Sinh_al = 16780746,
	GDK_Sinh_au = 16780694,
	GDK_Sinh_au2 = 16780766,
	GDK_Sinh_ba = 16780726,
	GDK_Sinh_bha = 16780727,
	GDK_Sinh_ca = 16780704,
	GDK_Sinh_cha = 16780705,
	GDK_Sinh_dda = 16780713,
	GDK_Sinh_ddha = 16780714,
	GDK_Sinh_dha = 16780719,
	GDK_Sinh_dhha = 16780720,
	GDK_Sinh_e = 16780689,
	GDK_Sinh_e2 = 16780761,
	GDK_Sinh_ee = 16780690,
	GDK_Sinh_ee2 = 16780762,
	GDK_Sinh_fa = 16780742,
	GDK_Sinh_ga = 16780700,
	GDK_Sinh_gha = 16780701,
	GDK_Sinh_h2 = 16780675,
	GDK_Sinh_ha = 16780740,
	GDK_Sinh_i = 16780681,
	GDK_Sinh_i2 = 16780754,
	GDK_Sinh_ii = 16780682,
	GDK_Sinh_ii2 = 16780755,
	GDK_Sinh_ja = 16780706,
	GDK_Sinh_jha = 16780707,
	GDK_Sinh_jnya = 16780709,
	GDK_Sinh_ka = 16780698,
	GDK_Sinh_kha = 16780699,
	GDK_Sinh_kunddaliya = 16780788,
	GDK_Sinh_la = 16780733,
	GDK_Sinh_lla = 16780741,
	GDK_Sinh_lu = 16780687,
	GDK_Sinh_lu2 = 16780767,
	GDK_Sinh_luu = 16780688,
	GDK_Sinh_luu2 = 16780787,
	GDK_Sinh_ma = 16780728,
	GDK_Sinh_mba = 16780729,
	GDK_Sinh_na = 16780721,
	GDK_Sinh_ndda = 16780716,
	GDK_Sinh_ndha = 16780723,
	GDK_Sinh_ng = 16780674,
	GDK_Sinh_ng2 = 16780702,
	GDK_Sinh_nga = 16780703,
	GDK_Sinh_nja = 16780710,
	GDK_Sinh_nna = 16780715,
	GDK_Sinh_nya = 16780708,
	GDK_Sinh_o = 16780692,
	GDK_Sinh_o2 = 16780764,
	GDK_Sinh_oo = 16780693,
	GDK_Sinh_oo2 = 16780765,
	GDK_Sinh_pa = 16780724,
	GDK_Sinh_pha = 16780725,
	GDK_Sinh_ra = 16780731,
	GDK_Sinh_ri = 16780685,
	GDK_Sinh_rii = 16780686,
	GDK_Sinh_ru2 = 16780760,
	GDK_Sinh_ruu2 = 16780786,
	GDK_Sinh_sa = 16780739,
	GDK_Sinh_sha = 16780737,
	GDK_Sinh_ssha = 16780738,
	GDK_Sinh_tha = 16780717,
	GDK_Sinh_thha = 16780718,
	GDK_Sinh_tta = 16780711,
	GDK_Sinh_ttha = 16780712,
	GDK_Sinh_u = 16780683,
	GDK_Sinh_u2 = 16780756,
	GDK_Sinh_uu = 16780684,
	GDK_Sinh_uu2 = 16780758,
	GDK_Sinh_va = 16780736,
	GDK_Sinh_ya = 16780730,
	GDK_Sleep = 269025071,
	GDK_SlowKeys_Enable = 65139,
	GDK_Spell = 269025148,
	GDK_SplitScreen = 269025149,
	GDK_Standby = 269025040,
	GDK_Start = 269025050,
	GDK_StickyKeys_Enable = 65141,
	GDK_Stop = 269025064,
	GDK_Subtitle = 269025178,
	GDK_Super_L = 65515,
	GDK_Super_R = 65516,
	GDK_Support = 269025150,
	GDK_Suspend = 269025191,
	GDK_Switch_VT_1 = 269024769,
	GDK_Switch_VT_10 = 269024778,
	GDK_Switch_VT_11 = 269024779,
	GDK_Switch_VT_12 = 269024780,
	GDK_Switch_VT_2 = 269024770,
	GDK_Switch_VT_3 = 269024771,
	GDK_Switch_VT_4 = 269024772,
	GDK_Switch_VT_5 = 269024773,
	GDK_Switch_VT_6 = 269024774,
	GDK_Switch_VT_7 = 269024775,
	GDK_Switch_VT_8 = 269024776,
	GDK_Switch_VT_9 = 269024777,
	GDK_Sys_Req = 65301,
	GDK_T = 84,
	GDK_THORN = 222,
	GDK_Tab = 65289,
	GDK_Tabovedot = 16785002,
	GDK_TaskPane = 269025151,
	GDK_Tcaron = 427,
	GDK_Tcedilla = 478,
	GDK_Terminal = 269025152,
	GDK_Terminate_Server = 65237,
	GDK_Thai_baht = 3551,
	GDK_Thai_bobaimai = 3514,
	GDK_Thai_chochan = 3496,
	GDK_Thai_chochang = 3498,
	GDK_Thai_choching = 3497,
	GDK_Thai_chochoe = 3500,
	GDK_Thai_dochada = 3502,
	GDK_Thai_dodek = 3508,
	GDK_Thai_fofa = 3517,
	GDK_Thai_fofan = 3519,
	GDK_Thai_hohip = 3531,
	GDK_Thai_honokhuk = 3534,
	GDK_Thai_khokhai = 3490,
	GDK_Thai_khokhon = 3493,
	GDK_Thai_khokhuat = 3491,
	GDK_Thai_khokhwai = 3492,
	GDK_Thai_khorakhang = 3494,
	GDK_Thai_kokai = 3489,
	GDK_Thai_lakkhangyao = 3557,
	GDK_Thai_lekchet = 3575,
	GDK_Thai_lekha = 3573,
	GDK_Thai_lekhok = 3574,
	GDK_Thai_lekkao = 3577,
	GDK_Thai_leknung = 3569,
	GDK_Thai_lekpaet = 3576,
	GDK_Thai_leksam = 3571,
	GDK_Thai_leksi = 3572,
	GDK_Thai_leksong = 3570,
	GDK_Thai_leksun = 3568,
	GDK_Thai_lochula = 3532,
	GDK_Thai_loling = 3525,
	GDK_Thai_lu = 3526,
	GDK_Thai_maichattawa = 3563,
	GDK_Thai_maiek = 3560,
	GDK_Thai_maihanakat = 3537,
	GDK_Thai_maihanakat_maitho = 3550,
	GDK_Thai_maitaikhu = 3559,
	GDK_Thai_maitho = 3561,
	GDK_Thai_maitri = 3562,
	GDK_Thai_maiyamok = 3558,
	GDK_Thai_moma = 3521,
	GDK_Thai_ngongu = 3495,
	GDK_Thai_nikhahit = 3565,
	GDK_Thai_nonen = 3507,
	GDK_Thai_nonu = 3513,
	GDK_Thai_oang = 3533,
	GDK_Thai_paiyannoi = 3535,
	GDK_Thai_phinthu = 3546,
	GDK_Thai_phophan = 3518,
	GDK_Thai_phophung = 3516,
	GDK_Thai_phosamphao = 3520,
	GDK_Thai_popla = 3515,
	GDK_Thai_rorua = 3523,
	GDK_Thai_ru = 3524,
	GDK_Thai_saraa = 3536,
	GDK_Thai_saraaa = 3538,
	GDK_Thai_saraae = 3553,
	GDK_Thai_saraaimaimalai = 3556,
	GDK_Thai_saraaimaimuan = 3555,
	GDK_Thai_saraam = 3539,
	GDK_Thai_sarae = 3552,
	GDK_Thai_sarai = 3540,
	GDK_Thai_saraii = 3541,
	GDK_Thai_sarao = 3554,
	GDK_Thai_sarau = 3544,
	GDK_Thai_saraue = 3542,
	GDK_Thai_sarauee = 3543,
	GDK_Thai_sarauu = 3545,
	GDK_Thai_sorusi = 3529,
	GDK_Thai_sosala = 3528,
	GDK_Thai_soso = 3499,
	GDK_Thai_sosua = 3530,
	GDK_Thai_thanthakhat = 3564,
	GDK_Thai_thonangmontho = 3505,
	GDK_Thai_thophuthao = 3506,
	GDK_Thai_thothahan = 3511,
	GDK_Thai_thothan = 3504,
	GDK_Thai_thothong = 3512,
	GDK_Thai_thothung = 3510,
	GDK_Thai_topatak = 3503,
	GDK_Thai_totao = 3509,
	GDK_Thai_wowaen = 3527,
	GDK_Thai_yoyak = 3522,
	GDK_Thai_yoying = 3501,
	GDK_Thorn = 222,
	GDK_Time = 269025183,
	GDK_ToDoList = 269025055,
	GDK_Tools = 269025153,
	GDK_TopMenu = 269025186,
	GDK_TouchpadOff = 269025201,
	GDK_TouchpadOn = 269025200,
	GDK_TouchpadToggle = 269025193,
	GDK_Touroku = 65323,
	GDK_Travel = 269025154,
	GDK_Tslash = 940,
	GDK_U = 85,
	GDK_UWB = 269025174,
	GDK_Uacute = 218,
	GDK_Ubelowdot = 16785124,
	GDK_Ubreve = 733,
	GDK_Ucircumflex = 219,
	GDK_Udiaeresis = 220,
	GDK_Udoubleacute = 475,
	GDK_Ugrave = 217,
	GDK_Uhook = 16785126,
	GDK_Uhorn = 16777647,
	GDK_Uhornacute = 16785128,
	GDK_Uhornbelowdot = 16785136,
	GDK_Uhorngrave = 16785130,
	GDK_Uhornhook = 16785132,
	GDK_Uhorntilde = 16785134,
	GDK_Ukrainian_GHE_WITH_UPTURN = 1725,
	GDK_Ukrainian_I = 1718,
	GDK_Ukrainian_IE = 1716,
	GDK_Ukrainian_YI = 1719,
	GDK_Ukrainian_ghe_with_upturn = 1709,
	GDK_Ukrainian_i = 1702,
	GDK_Ukrainian_ie = 1700,
	GDK_Ukrainian_yi = 1703,
	GDK_Ukranian_I = 1718,
	GDK_Ukranian_JE = 1716,
	GDK_Ukranian_YI = 1719,
	GDK_Ukranian_i = 1702,
	GDK_Ukranian_je = 1700,
	GDK_Ukranian_yi = 1703,
	GDK_Umacron = 990,
	GDK_Undo = 65381,
	GDK_Ungrab = 269024800,
	GDK_Uogonek = 985,
	GDK_Up = 65362,
	GDK_Uring = 473,
	GDK_User1KB = 269025157,
	GDK_User2KB = 269025158,
	GDK_UserPB = 269025156,
	GDK_Utilde = 989,
	GDK_V = 86,
	GDK_VendorHome = 269025076,
	GDK_Video = 269025159,
	GDK_View = 269025185,
	GDK_VoidSymbol = 16777215,
	GDK_W = 87,
	GDK_WLAN = 269025173,
	GDK_WWW = 269025070,
	GDK_Wacute = 16785026,
	GDK_WakeUp = 269025067,
	GDK_Wcircumflex = 16777588,
	GDK_Wdiaeresis = 16785028,
	GDK_WebCam = 269025167,
	GDK_Wgrave = 16785024,
	GDK_WheelButton = 269025160,
	GDK_WindowClear = 269025109,
	GDK_WonSign = 16785577,
	GDK_Word = 269025161,
	GDK_X = 88,
	GDK_Xabovedot = 16785034,
	GDK_Xfer = 269025162,
	GDK_Y = 89,
	GDK_Yacute = 221,
	GDK_Ybelowdot = 16785140,
	GDK_Ycircumflex = 16777590,
	GDK_Ydiaeresis = 5054,
	GDK_Yellow = 269025189,
	GDK_Ygrave = 16785138,
	GDK_Yhook = 16785142,
	GDK_Ytilde = 16785144,
	GDK_Z = 90,
	GDK_Zabovedot = 431,
	GDK_Zacute = 428,
	GDK_Zcaron = 430,
	GDK_Zen_Koho = 65341,
	GDK_Zenkaku = 65320,
	GDK_Zenkaku_Hankaku = 65322,
	GDK_ZoomIn = 269025163,
	GDK_ZoomOut = 269025164,
	GDK_Zstroke = 16777653,
	GDK_a = 97,
	GDK_aacute = 225,
	GDK_abelowdot = 16785057,
	GDK_abovedot = 511,
	GDK_abreve = 483,
	GDK_abreveacute = 16785071,
	GDK_abrevebelowdot = 16785079,
	GDK_abrevegrave = 16785073,
	GDK_abrevehook = 16785075,
	GDK_abrevetilde = 16785077,
	GDK_acircumflex = 226,
	GDK_acircumflexacute = 16785061,
	GDK_acircumflexbelowdot = 16785069,
	GDK_acircumflexgrave = 16785063,
	GDK_acircumflexhook = 16785065,
	GDK_acircumflextilde = 16785067,
	GDK_acute = 180,
	GDK_adiaeresis = 228,
	GDK_ae = 230,
	GDK_agrave = 224,
	GDK_ahook = 16785059,
	GDK_amacron = 992,
	GDK_ampersand = 38,
	GDK_aogonek = 433,
	GDK_apostrophe = 39,
	GDK_approxeq = 16785992,
	GDK_approximate = 2248,
	GDK_aring = 229,
	GDK_asciicircum = 94,
	GDK_asciitilde = 126,
	GDK_asterisk = 42,
	GDK_at = 64,
	GDK_atilde = 227,
	GDK_b = 98,
	GDK_babovedot = 16784899,
	GDK_backslash = 92,
	GDK_ballotcross = 2804,
	GDK_bar = 124,
	GDK_because = 16785973,
	GDK_blank = 2527,
	GDK_botintegral = 2213,
	GDK_botleftparens = 2220,
	GDK_botleftsqbracket = 2216,
	GDK_botleftsummation = 2226,
	GDK_botrightparens = 2222,
	GDK_botrightsqbracket = 2218,
	GDK_botrightsummation = 2230,
	GDK_bott = 2550,
	GDK_botvertsummationconnector = 2228,
	GDK_braceleft = 123,
	GDK_braceright = 125,
	GDK_bracketleft = 91,
	GDK_bracketright = 93,
	GDK_braille_blank = 16787456,
	GDK_braille_dot_1 = 65521,
	GDK_braille_dot_10 = 65530,
	GDK_braille_dot_2 = 65522,
	GDK_braille_dot_3 = 65523,
	GDK_braille_dot_4 = 65524,
	GDK_braille_dot_5 = 65525,
	GDK_braille_dot_6 = 65526,
	GDK_braille_dot_7 = 65527,
	GDK_braille_dot_8 = 65528,
	GDK_braille_dot_9 = 65529,
	GDK_braille_dots_1 = 16787457,
	GDK_braille_dots_12 = 16787459,
	GDK_braille_dots_123 = 16787463,
	GDK_braille_dots_1234 = 16787471,
	GDK_braille_dots_12345 = 16787487,
	GDK_braille_dots_123456 = 16787519,
	GDK_braille_dots_1234567 = 16787583,
	GDK_braille_dots_12345678 = 16787711,
	GDK_braille_dots_1234568 = 16787647,
	GDK_braille_dots_123457 = 16787551,
	GDK_braille_dots_1234578 = 16787679,
	GDK_braille_dots_123458 = 16787615,
	GDK_braille_dots_12346 = 16787503,
	GDK_braille_dots_123467 = 16787567,
	GDK_braille_dots_1234678 = 16787695,
	GDK_braille_dots_123468 = 16787631,
	GDK_braille_dots_12347 = 16787535,
	GDK_braille_dots_123478 = 16787663,
	GDK_braille_dots_12348 = 16787599,
	GDK_braille_dots_1235 = 16787479,
	GDK_braille_dots_12356 = 16787511,
	GDK_braille_dots_123567 = 16787575,
	GDK_braille_dots_1235678 = 16787703,
	GDK_braille_dots_123568 = 16787639,
	GDK_braille_dots_12357 = 16787543,
	GDK_braille_dots_123578 = 16787671,
	GDK_braille_dots_12358 = 16787607,
	GDK_braille_dots_1236 = 16787495,
	GDK_braille_dots_12367 = 16787559,
	GDK_braille_dots_123678 = 16787687,
	GDK_braille_dots_12368 = 16787623,
	GDK_braille_dots_1237 = 16787527,
	GDK_braille_dots_12378 = 16787655,
	GDK_braille_dots_1238 = 16787591,
	GDK_braille_dots_124 = 16787467,
	GDK_braille_dots_1245 = 16787483,
	GDK_braille_dots_12456 = 16787515,
	GDK_braille_dots_124567 = 16787579,
	GDK_braille_dots_1245678 = 16787707,
	GDK_braille_dots_124568 = 16787643,
	GDK_braille_dots_12457 = 16787547,
	GDK_braille_dots_124578 = 16787675,
	GDK_braille_dots_12458 = 16787611,
	GDK_braille_dots_1246 = 16787499,
	GDK_braille_dots_12467 = 16787563,
	GDK_braille_dots_124678 = 16787691,
	GDK_braille_dots_12468 = 16787627,
	GDK_braille_dots_1247 = 16787531,
	GDK_braille_dots_12478 = 16787659,
	GDK_braille_dots_1248 = 16787595,
	GDK_braille_dots_125 = 16787475,
	GDK_braille_dots_1256 = 16787507,
	GDK_braille_dots_12567 = 16787571,
	GDK_braille_dots_125678 = 16787699,
	GDK_braille_dots_12568 = 16787635,
	GDK_braille_dots_1257 = 16787539,
	GDK_braille_dots_12578 = 16787667,
	GDK_braille_dots_1258 = 16787603,
	GDK_braille_dots_126 = 16787491,
	GDK_braille_dots_1267 = 16787555,
	GDK_braille_dots_12678 = 16787683,
	GDK_braille_dots_1268 = 16787619,
	GDK_braille_dots_127 = 16787523,
	GDK_braille_dots_1278 = 16787651,
	GDK_braille_dots_128 = 16787587,
	GDK_braille_dots_13 = 16787461,
	GDK_braille_dots_134 = 16787469,
	GDK_braille_dots_1345 = 16787485,
	GDK_braille_dots_13456 = 16787517,
	GDK_braille_dots_134567 = 16787581,
	GDK_braille_dots_1345678 = 16787709,
	GDK_braille_dots_134568 = 16787645,
	GDK_braille_dots_13457 = 16787549,
	GDK_braille_dots_134578 = 16787677,
	GDK_braille_dots_13458 = 16787613,
	GDK_braille_dots_1346 = 16787501,
	GDK_braille_dots_13467 = 16787565,
	GDK_braille_dots_134678 = 16787693,
	GDK_braille_dots_13468 = 16787629,
	GDK_braille_dots_1347 = 16787533,
	GDK_braille_dots_13478 = 16787661,
	GDK_braille_dots_1348 = 16787597,
	GDK_braille_dots_135 = 16787477,
	GDK_braille_dots_1356 = 16787509,
	GDK_braille_dots_13567 = 16787573,
	GDK_braille_dots_135678 = 16787701,
	GDK_braille_dots_13568 = 16787637,
	GDK_braille_dots_1357 = 16787541,
	GDK_braille_dots_13578 = 16787669,
	GDK_braille_dots_1358 = 16787605,
	GDK_braille_dots_136 = 16787493,
	GDK_braille_dots_1367 = 16787557,
	GDK_braille_dots_13678 = 16787685,
	GDK_braille_dots_1368 = 16787621,
	GDK_braille_dots_137 = 16787525,
	GDK_braille_dots_1378 = 16787653,
	GDK_braille_dots_138 = 16787589,
	GDK_braille_dots_14 = 16787465,
	GDK_braille_dots_145 = 16787481,
	GDK_braille_dots_1456 = 16787513,
	GDK_braille_dots_14567 = 16787577,
	GDK_braille_dots_145678 = 16787705,
	GDK_braille_dots_14568 = 16787641,
	GDK_braille_dots_1457 = 16787545,
	GDK_braille_dots_14578 = 16787673,
	GDK_braille_dots_1458 = 16787609,
	GDK_braille_dots_146 = 16787497,
	GDK_braille_dots_1467 = 16787561,
	GDK_braille_dots_14678 = 16787689,
	GDK_braille_dots_1468 = 16787625,
	GDK_braille_dots_147 = 16787529,
	GDK_braille_dots_1478 = 16787657,
	GDK_braille_dots_148 = 16787593,
	GDK_braille_dots_15 = 16787473,
	GDK_braille_dots_156 = 16787505,
	GDK_braille_dots_1567 = 16787569,
	GDK_braille_dots_15678 = 16787697,
	GDK_braille_dots_1568 = 16787633,
	GDK_braille_dots_157 = 16787537,
	GDK_braille_dots_1578 = 16787665,
	GDK_braille_dots_158 = 16787601,
	GDK_braille_dots_16 = 16787489,
	GDK_braille_dots_167 = 16787553,
	GDK_braille_dots_1678 = 16787681,
	GDK_braille_dots_168 = 16787617,
	GDK_braille_dots_17 = 16787521,
	GDK_braille_dots_178 = 16787649,
	GDK_braille_dots_18 = 16787585,
	GDK_braille_dots_2 = 16787458,
	GDK_braille_dots_23 = 16787462,
	GDK_braille_dots_234 = 16787470,
	GDK_braille_dots_2345 = 16787486,
	GDK_braille_dots_23456 = 16787518,
	GDK_braille_dots_234567 = 16787582,
	GDK_braille_dots_2345678 = 16787710,
	GDK_braille_dots_234568 = 16787646,
	GDK_braille_dots_23457 = 16787550,
	GDK_braille_dots_234578 = 16787678,
	GDK_braille_dots_23458 = 16787614,
	GDK_braille_dots_2346 = 16787502,
	GDK_braille_dots_23467 = 16787566,
	GDK_braille_dots_234678 = 16787694,
	GDK_braille_dots_23468 = 16787630,
	GDK_braille_dots_2347 = 16787534,
	GDK_braille_dots_23478 = 16787662,
	GDK_braille_dots_2348 = 16787598,
	GDK_braille_dots_235 = 16787478,
	GDK_braille_dots_2356 = 16787510,
	GDK_braille_dots_23567 = 16787574,
	GDK_braille_dots_235678 = 16787702,
	GDK_braille_dots_23568 = 16787638,
	GDK_braille_dots_2357 = 16787542,
	GDK_braille_dots_23578 = 16787670,
	GDK_braille_dots_2358 = 16787606,
	GDK_braille_dots_236 = 16787494,
	GDK_braille_dots_2367 = 16787558,
	GDK_braille_dots_23678 = 16787686,
	GDK_braille_dots_2368 = 16787622,
	GDK_braille_dots_237 = 16787526,
	GDK_braille_dots_2378 = 16787654,
	GDK_braille_dots_238 = 16787590,
	GDK_braille_dots_24 = 16787466,
	GDK_braille_dots_245 = 16787482,
	GDK_braille_dots_2456 = 16787514,
	GDK_braille_dots_24567 = 16787578,
	GDK_braille_dots_245678 = 16787706,
	GDK_braille_dots_24568 = 16787642,
	GDK_braille_dots_2457 = 16787546,
	GDK_braille_dots_24578 = 16787674,
	GDK_braille_dots_2458 = 16787610,
	GDK_braille_dots_246 = 16787498,
	GDK_braille_dots_2467 = 16787562,
	GDK_braille_dots_24678 = 16787690,
	GDK_braille_dots_2468 = 16787626,
	GDK_braille_dots_247 = 16787530,
	GDK_braille_dots_2478 = 16787658,
	GDK_braille_dots_248 = 16787594,
	GDK_braille_dots_25 = 16787474,
	GDK_braille_dots_256 = 16787506,
	GDK_braille_dots_2567 = 16787570,
	GDK_braille_dots_25678 = 16787698,
	GDK_braille_dots_2568 = 16787634,
	GDK_braille_dots_257 = 16787538,
	GDK_braille_dots_2578 = 16787666,
	GDK_braille_dots_258 = 16787602,
	GDK_braille_dots_26 = 16787490,
	GDK_braille_dots_267 = 16787554,
	GDK_braille_dots_2678 = 16787682,
	GDK_braille_dots_268 = 16787618,
	GDK_braille_dots_27 = 16787522,
	GDK_braille_dots_278 = 16787650,
	GDK_braille_dots_28 = 16787586,
	GDK_braille_dots_3 = 16787460,
	GDK_braille_dots_34 = 16787468,
	GDK_braille_dots_345 = 16787484,
	GDK_braille_dots_3456 = 16787516,
	GDK_braille_dots_34567 = 16787580,
	GDK_braille_dots_345678 = 16787708,
	GDK_braille_dots_34568 = 16787644,
	GDK_braille_dots_3457 = 16787548,
	GDK_braille_dots_34578 = 16787676,
	GDK_braille_dots_3458 = 16787612,
	GDK_braille_dots_346 = 16787500,
	GDK_braille_dots_3467 = 16787564,
	GDK_braille_dots_34678 = 16787692,
	GDK_braille_dots_3468 = 16787628,
	GDK_braille_dots_347 = 16787532,
	GDK_braille_dots_3478 = 16787660,
	GDK_braille_dots_348 = 16787596,
	GDK_braille_dots_35 = 16787476,
	GDK_braille_dots_356 = 16787508,
	GDK_braille_dots_3567 = 16787572,
	GDK_braille_dots_35678 = 16787700,
	GDK_braille_dots_3568 = 16787636,
	GDK_braille_dots_357 = 16787540,
	GDK_braille_dots_3578 = 16787668,
	GDK_braille_dots_358 = 16787604,
	GDK_braille_dots_36 = 16787492,
	GDK_braille_dots_367 = 16787556,
	GDK_braille_dots_3678 = 16787684,
	GDK_braille_dots_368 = 16787620,
	GDK_braille_dots_37 = 16787524,
	GDK_braille_dots_378 = 16787652,
	GDK_braille_dots_38 = 16787588,
	GDK_braille_dots_4 = 16787464,
	GDK_braille_dots_45 = 16787480,
	GDK_braille_dots_456 = 16787512,
	GDK_braille_dots_4567 = 16787576,
	GDK_braille_dots_45678 = 16787704,
	GDK_braille_dots_4568 = 16787640,
	GDK_braille_dots_457 = 16787544,
	GDK_braille_dots_4578 = 16787672,
	GDK_braille_dots_458 = 16787608,
	GDK_braille_dots_46 = 16787496,
	GDK_braille_dots_467 = 16787560,
	GDK_braille_dots_4678 = 16787688,
	GDK_braille_dots_468 = 16787624,
	GDK_braille_dots_47 = 16787528,
	GDK_braille_dots_478 = 16787656,
	GDK_braille_dots_48 = 16787592,
	GDK_braille_dots_5 = 16787472,
	GDK_braille_dots_56 = 16787504,
	GDK_braille_dots_567 = 16787568,
	GDK_braille_dots_5678 = 16787696,
	GDK_braille_dots_568 = 16787632,
	GDK_braille_dots_57 = 16787536,
	GDK_braille_dots_578 = 16787664,
	GDK_braille_dots_58 = 16787600,
	GDK_braille_dots_6 = 16787488,
	GDK_braille_dots_67 = 16787552,
	GDK_braille_dots_678 = 16787680,
	GDK_braille_dots_68 = 16787616,
	GDK_braille_dots_7 = 16787520,
	GDK_braille_dots_78 = 16787648,
	GDK_braille_dots_8 = 16787584,
	GDK_breve = 418,
	GDK_brokenbar = 166,
	GDK_c = 99,
	GDK_c_h = 65187,
	GDK_cabovedot = 741,
	GDK_cacute = 486,
	GDK_careof = 2744,
	GDK_caret = 2812,
	GDK_caron = 439,
	GDK_ccaron = 488,
	GDK_ccedilla = 231,
	GDK_ccircumflex = 742,
	GDK_cedilla = 184,
	GDK_cent = 162,
	GDK_ch = 65184,
	GDK_checkerboard = 2529,
	GDK_checkmark = 2803,
	GDK_circle = 3023,
	GDK_club = 2796,
	GDK_colon = 58,
	GDK_comma = 44,
	GDK_containsas = 16785931,
	GDK_copyright = 169,
	GDK_cr = 2532,
	GDK_crossinglines = 2542,
	GDK_cuberoot = 16785947,
	GDK_currency = 164,
	GDK_cursor = 2815,
	GDK_d = 100,
	GDK_dabovedot = 16784907,
	GDK_dagger = 2801,
	GDK_dcaron = 495,
	GDK_dead_A = 65153,
	GDK_dead_E = 65155,
	GDK_dead_I = 65157,
	GDK_dead_O = 65159,
	GDK_dead_U = 65161,
	GDK_dead_a = 65152,
	GDK_dead_abovecomma = 65124,
	GDK_dead_abovedot = 65110,
	GDK_dead_abovereversedcomma = 65125,
	GDK_dead_abovering = 65112,
	GDK_dead_acute = 65105,
	GDK_dead_belowbreve = 65131,
	GDK_dead_belowcircumflex = 65129,
	GDK_dead_belowcomma = 65134,
	GDK_dead_belowdiaeresis = 65132,
	GDK_dead_belowdot = 65120,
	GDK_dead_belowmacron = 65128,
	GDK_dead_belowring = 65127,
	GDK_dead_belowtilde = 65130,
	GDK_dead_breve = 65109,
	GDK_dead_capital_schwa = 65163,
	GDK_dead_caron = 65114,
	GDK_dead_cedilla = 65115,
	GDK_dead_circumflex = 65106,
	GDK_dead_currency = 65135,
	GDK_dead_dasia = 65125,
	GDK_dead_diaeresis = 65111,
	GDK_dead_doubleacute = 65113,
	GDK_dead_doublegrave = 65126,
	GDK_dead_e = 65154,
	GDK_dead_grave = 65104,
	GDK_dead_greek = 65164,
	GDK_dead_hook = 65121,
	GDK_dead_horn = 65122,
	GDK_dead_i = 65156,
	GDK_dead_invertedbreve = 65133,
	GDK_dead_iota = 65117,
	GDK_dead_macron = 65108,
	GDK_dead_o = 65158,
	GDK_dead_ogonek = 65116,
	GDK_dead_perispomeni = 65107,
	GDK_dead_psili = 65124,
	GDK_dead_semivoiced_sound = 65119,
	GDK_dead_small_schwa = 65162,
	GDK_dead_stroke = 65123,
	GDK_dead_tilde = 65107,
	GDK_dead_u = 65160,
	GDK_dead_voiced_sound = 65118,
	GDK_decimalpoint = 2749,
	GDK_degree = 176,
	GDK_diaeresis = 168,
	GDK_diamond = 2797,
	GDK_digitspace = 2725,
	GDK_dintegral = 16785964,
	GDK_division = 247,
	GDK_dollar = 36,
	GDK_doubbaselinedot = 2735,
	GDK_doubleacute = 445,
	GDK_doubledagger = 2802,
	GDK_doublelowquotemark = 2814,
	GDK_downarrow = 2302,
	GDK_downcaret = 2984,
	GDK_downshoe = 3030,
	GDK_downstile = 3012,
	GDK_downtack = 3010,
	GDK_dstroke = 496,
	GDK_e = 101,
	GDK_eabovedot = 1004,
	GDK_eacute = 233,
	GDK_ebelowdot = 16785081,
	GDK_ecaron = 492,
	GDK_ecircumflex = 234,
	GDK_ecircumflexacute = 16785087,
	GDK_ecircumflexbelowdot = 16785095,
	GDK_ecircumflexgrave = 16785089,
	GDK_ecircumflexhook = 16785091,
	GDK_ecircumflextilde = 16785093,
	GDK_ediaeresis = 235,
	GDK_egrave = 232,
	GDK_ehook = 16785083,
	GDK_eightsubscript = 16785544,
	GDK_eightsuperior = 16785528,
	GDK_elementof = 16785928,
	GDK_ellipsis = 2734,
	GDK_em3space = 2723,
	GDK_em4space = 2724,
	GDK_emacron = 954,
	GDK_emdash = 2729,
	GDK_emfilledcircle = 2782,
	GDK_emfilledrect = 2783,
	GDK_emopencircle = 2766,
	GDK_emopenrectangle = 2767,
	GDK_emptyset = 16785925,
	GDK_emspace = 2721,
	GDK_endash = 2730,
	GDK_enfilledcircbullet = 2790,
	GDK_enfilledsqbullet = 2791,
	GDK_eng = 959,
	GDK_enopencircbullet = 2784,
	GDK_enopensquarebullet = 2785,
	GDK_enspace = 2722,
	GDK_eogonek = 490,
	GDK_equal = 61,
	GDK_eth = 240,
	GDK_etilde = 16785085,
	GDK_exclam = 33,
	GDK_exclamdown = 161,
	GDK_ezh = 16777874,
	GDK_f = 102,
	GDK_fabovedot = 16784927,
	GDK_femalesymbol = 2808,
	GDK_ff = 2531,
	GDK_figdash = 2747,
	GDK_filledlefttribullet = 2780,
	GDK_filledrectbullet = 2779,
	GDK_filledrighttribullet = 2781,
	GDK_filledtribulletdown = 2793,
	GDK_filledtribulletup = 2792,
	GDK_fiveeighths = 2757,
	GDK_fivesixths = 2743,
	GDK_fivesubscript = 16785541,
	GDK_fivesuperior = 16785525,
	GDK_fourfifths = 2741,
	GDK_foursubscript = 16785540,
	GDK_foursuperior = 16785524,
	GDK_fourthroot = 16785948,
	GDK_function = 2294,
	GDK_g = 103,
	GDK_gabovedot = 757,
	GDK_gbreve = 699,
	GDK_gcaron = 16777703,
	GDK_gcedilla = 955,
	GDK_gcircumflex = 760,
	GDK_grave = 96,
	GDK_greater = 62,
	GDK_greaterthanequal = 2238,
	GDK_guillemotleft = 171,
	GDK_guillemotright = 187,
	GDK_h = 104,
	GDK_hairspace = 2728,
	GDK_hcircumflex = 694,
	GDK_heart = 2798,
	GDK_hebrew_aleph = 3296,
	GDK_hebrew_ayin = 3314,
	GDK_hebrew_bet = 3297,
	GDK_hebrew_beth = 3297,
	GDK_hebrew_chet = 3303,
	GDK_hebrew_dalet = 3299,
	GDK_hebrew_daleth = 3299,
	GDK_hebrew_doublelowline = 3295,
	GDK_hebrew_finalkaph = 3306,
	GDK_hebrew_finalmem = 3309,
	GDK_hebrew_finalnun = 3311,
	GDK_hebrew_finalpe = 3315,
	GDK_hebrew_finalzade = 3317,
	GDK_hebrew_finalzadi = 3317,
	GDK_hebrew_gimel = 3298,
	GDK_hebrew_gimmel = 3298,
	GDK_hebrew_he = 3300,
	GDK_hebrew_het = 3303,
	GDK_hebrew_kaph = 3307,
	GDK_hebrew_kuf = 3319,
	GDK_hebrew_lamed = 3308,
	GDK_hebrew_mem = 3310,
	GDK_hebrew_nun = 3312,
	GDK_hebrew_pe = 3316,
	GDK_hebrew_qoph = 3319,
	GDK_hebrew_resh = 3320,
	GDK_hebrew_samech = 3313,
	GDK_hebrew_samekh = 3313,
	GDK_hebrew_shin = 3321,
	GDK_hebrew_taf = 3322,
	GDK_hebrew_taw = 3322,
	GDK_hebrew_tet = 3304,
	GDK_hebrew_teth = 3304,
	GDK_hebrew_waw = 3301,
	GDK_hebrew_yod = 3305,
	GDK_hebrew_zade = 3318,
	GDK_hebrew_zadi = 3318,
	GDK_hebrew_zain = 3302,
	GDK_hebrew_zayin = 3302,
	GDK_hexagram = 2778,
	GDK_horizconnector = 2211,
	GDK_horizlinescan1 = 2543,
	GDK_horizlinescan3 = 2544,
	GDK_horizlinescan5 = 2545,
	GDK_horizlinescan7 = 2546,
	GDK_horizlinescan9 = 2547,
	GDK_hstroke = 689,
	GDK_ht = 2530,
	GDK_hyphen = 173,
	GDK_i = 105,
	GDK_iTouch = 269025120,
	GDK_iacute = 237,
	GDK_ibelowdot = 16785099,
	GDK_ibreve = 16777517,
	GDK_icircumflex = 238,
	GDK_identical = 2255,
	GDK_idiaeresis = 239,
	GDK_idotless = 697,
	GDK_ifonlyif = 2253,
	GDK_igrave = 236,
	GDK_ihook = 16785097,
	GDK_imacron = 1007,
	GDK_implies = 2254,
	GDK_includedin = 2266,
	GDK_includes = 2267,
	GDK_infinity = 2242,
	GDK_integral = 2239,
	GDK_intersection = 2268,
	GDK_iogonek = 999,
	GDK_itilde = 949,
	GDK_j = 106,
	GDK_jcircumflex = 700,
	GDK_jot = 3018,
	GDK_k = 107,
	GDK_kana_A = 1201,
	GDK_kana_CHI = 1217,
	GDK_kana_E = 1204,
	GDK_kana_FU = 1228,
	GDK_kana_HA = 1226,
	GDK_kana_HE = 1229,
	GDK_kana_HI = 1227,
	GDK_kana_HO = 1230,
	GDK_kana_HU = 1228,
	GDK_kana_I = 1202,
	GDK_kana_KA = 1206,
	GDK_kana_KE = 1209,
	GDK_kana_KI = 1207,
	GDK_kana_KO = 1210,
	GDK_kana_KU = 1208,
	GDK_kana_MA = 1231,
	GDK_kana_ME = 1234,
	GDK_kana_MI = 1232,
	GDK_kana_MO = 1235,
	GDK_kana_MU = 1233,
	GDK_kana_N = 1245,
	GDK_kana_NA = 1221,
	GDK_kana_NE = 1224,
	GDK_kana_NI = 1222,
	GDK_kana_NO = 1225,
	GDK_kana_NU = 1223,
	GDK_kana_O = 1205,
	GDK_kana_RA = 1239,
	GDK_kana_RE = 1242,
	GDK_kana_RI = 1240,
	GDK_kana_RO = 1243,
	GDK_kana_RU = 1241,
	GDK_kana_SA = 1211,
	GDK_kana_SE = 1214,
	GDK_kana_SHI = 1212,
	GDK_kana_SO = 1215,
	GDK_kana_SU = 1213,
	GDK_kana_TA = 1216,
	GDK_kana_TE = 1219,
	GDK_kana_TI = 1217,
	GDK_kana_TO = 1220,
	GDK_kana_TSU = 1218,
	GDK_kana_TU = 1218,
	GDK_kana_U = 1203,
	GDK_kana_WA = 1244,
	GDK_kana_WO = 1190,
	GDK_kana_YA = 1236,
	GDK_kana_YO = 1238,
	GDK_kana_YU = 1237,
	GDK_kana_a = 1191,
	GDK_kana_closingbracket = 1187,
	GDK_kana_comma = 1188,
	GDK_kana_conjunctive = 1189,
	GDK_kana_e = 1194,
	GDK_kana_fullstop = 1185,
	GDK_kana_i = 1192,
	GDK_kana_middledot = 1189,
	GDK_kana_o = 1195,
	GDK_kana_openingbracket = 1186,
	GDK_kana_switch = 65406,
	GDK_kana_tsu = 1199,
	GDK_kana_tu = 1199,
	GDK_kana_u = 1193,
	GDK_kana_ya = 1196,
	GDK_kana_yo = 1198,
	GDK_kana_yu = 1197,
	GDK_kappa = 930,
	GDK_kcedilla = 1011,
	GDK_kra = 930,
	GDK_l = 108,
	GDK_lacute = 485,
	GDK_latincross = 2777,
	GDK_lbelowdot = 16784951,
	GDK_lcaron = 437,
	GDK_lcedilla = 950,
	GDK_leftanglebracket = 2748,
	GDK_leftarrow = 2299,
	GDK_leftcaret = 2979,
	GDK_leftdoublequotemark = 2770,
	GDK_leftmiddlecurlybrace = 2223,
	GDK_leftopentriangle = 2764,
	GDK_leftpointer = 2794,
	GDK_leftradical = 2209,
	GDK_leftshoe = 3034,
	GDK_leftsinglequotemark = 2768,
	GDK_leftt = 2548,
	GDK_lefttack = 3036,
	GDK_less = 60,
	GDK_lessthanequal = 2236,
	GDK_lf = 2533,
	GDK_logicaland = 2270,
	GDK_logicalor = 2271,
	GDK_lowleftcorner = 2541,
	GDK_lowrightcorner = 2538,
	GDK_lstroke = 435,
	GDK_m = 109,
	GDK_mabovedot = 16784961,
	GDK_macron = 175,
	GDK_malesymbol = 2807,
	GDK_maltesecross = 2800,
	GDK_marker = 2751,
	GDK_masculine = 186,
	GDK_minus = 45,
	GDK_minutes = 2774,
	GDK_mu = 181,
	GDK_multiply = 215,
	GDK_musicalflat = 2806,
	GDK_musicalsharp = 2805,
	GDK_n = 110,
	GDK_nabla = 2245,
	GDK_nacute = 497,
	GDK_ncaron = 498,
	GDK_ncedilla = 1009,
	GDK_ninesubscript = 16785545,
	GDK_ninesuperior = 16785529,
	GDK_nl = 2536,
	GDK_nobreakspace = 160,
	GDK_notapproxeq = 16785991,
	GDK_notelementof = 16785929,
	GDK_notequal = 2237,
	GDK_notidentical = 16786018,
	GDK_notsign = 172,
	GDK_ntilde = 241,
	GDK_numbersign = 35,
	GDK_numerosign = 1712,
	GDK_o = 111,
	GDK_oacute = 243,
	GDK_obarred = 16777845,
	GDK_obelowdot = 16785101,
	GDK_ocaron = 16777682,
	GDK_ocircumflex = 244,
	GDK_ocircumflexacute = 16785105,
	GDK_ocircumflexbelowdot = 16785113,
	GDK_ocircumflexgrave = 16785107,
	GDK_ocircumflexhook = 16785109,
	GDK_ocircumflextilde = 16785111,
	GDK_odiaeresis = 246,
	GDK_odoubleacute = 501,
	GDK_oe = 5053,
	GDK_ogonek = 434,
	GDK_ograve = 242,
	GDK_ohook = 16785103,
	GDK_ohorn = 16777633,
	GDK_ohornacute = 16785115,
	GDK_ohornbelowdot = 16785123,
	GDK_ohorngrave = 16785117,
	GDK_ohornhook = 16785119,
	GDK_ohorntilde = 16785121,
	GDK_omacron = 1010,
	GDK_oneeighth = 2755,
	GDK_onefifth = 2738,
	GDK_onehalf = 189,
	GDK_onequarter = 188,
	GDK_onesixth = 2742,
	GDK_onesubscript = 16785537,
	GDK_onesuperior = 185,
	GDK_onethird = 2736,
	GDK_ooblique = 248,
	GDK_openrectbullet = 2786,
	GDK_openstar = 2789,
	GDK_opentribulletdown = 2788,
	GDK_opentribulletup = 2787,
	GDK_ordfeminine = 170,
	GDK_oslash = 248,
	GDK_otilde = 245,
	GDK_overbar = 3008,
	GDK_overline = 1150,
	GDK_p = 112,
	GDK_pabovedot = 16784983,
	GDK_paragraph = 182,
	GDK_parenleft = 40,
	GDK_parenright = 41,
	GDK_partdifferential = 16785922,
	GDK_partialderivative = 2287,
	GDK_percent = 37,
	GDK_period = 46,
	GDK_periodcentered = 183,
	GDK_permille = 2773,
	GDK_phonographcopyright = 2811,
	GDK_plus = 43,
	GDK_plusminus = 177,
	GDK_prescription = 2772,
	GDK_prolongedsound = 1200,
	GDK_punctspace = 2726,
	GDK_q = 113,
	GDK_quad = 3020,
	GDK_question = 63,
	GDK_questiondown = 191,
	GDK_quotedbl = 34,
	GDK_quoteleft = 96,
	GDK_quoteright = 39,
	GDK_r = 114,
	GDK_racute = 480,
	GDK_radical = 2262,
	GDK_rcaron = 504,
	GDK_rcedilla = 947,
	GDK_registered = 174,
	GDK_rightanglebracket = 2750,
	GDK_rightarrow = 2301,
	GDK_rightcaret = 2982,
	GDK_rightdoublequotemark = 2771,
	GDK_rightmiddlecurlybrace = 2224,
	GDK_rightmiddlesummation = 2231,
	GDK_rightopentriangle = 2765,
	GDK_rightpointer = 2795,
	GDK_rightshoe = 3032,
	GDK_rightsinglequotemark = 2769,
	GDK_rightt = 2549,
	GDK_righttack = 3068,
	GDK_s = 115,
	GDK_sabovedot = 16784993,
	GDK_sacute = 438,
	GDK_scaron = 441,
	GDK_scedilla = 442,
	GDK_schwa = 16777817,
	GDK_scircumflex = 766,
	GDK_script_switch = 65406,
	GDK_seconds = 2775,
	GDK_section = 167,
	GDK_semicolon = 59,
	GDK_semivoicedsound = 1247,
	GDK_seveneighths = 2758,
	GDK_sevensubscript = 16785543,
	GDK_sevensuperior = 16785527,
	GDK_signaturemark = 2762,
	GDK_signifblank = 2732,
	GDK_similarequal = 2249,
	GDK_singlelowquotemark = 2813,
	GDK_sixsubscript = 16785542,
	GDK_sixsuperior = 16785526,
	GDK_slash = 47,
	GDK_soliddiamond = 2528,
	GDK_space = 32,
	GDK_squareroot = 16785946,
	GDK_ssharp = 223,
	GDK_sterling = 163,
	GDK_stricteq = 16786019,
	GDK_t = 116,
	GDK_tabovedot = 16785003,
	GDK_tcaron = 443,
	GDK_tcedilla = 510,
	GDK_telephone = 2809,
	GDK_telephonerecorder = 2810,
	GDK_therefore = 2240,
	GDK_thinspace = 2727,
	GDK_thorn = 254,
	GDK_threeeighths = 2756,
	GDK_threefifths = 2740,
	GDK_threequarters = 190,
	GDK_threesubscript = 16785539,
	GDK_threesuperior = 179,
	GDK_tintegral = 16785965,
	GDK_topintegral = 2212,
	GDK_topleftparens = 2219,
	GDK_topleftradical = 2210,
	GDK_topleftsqbracket = 2215,
	GDK_topleftsummation = 2225,
	GDK_toprightparens = 2221,
	GDK_toprightsqbracket = 2217,
	GDK_toprightsummation = 2229,
	GDK_topt = 2551,
	GDK_topvertsummationconnector = 2227,
	GDK_trademark = 2761,
	GDK_trademarkincircle = 2763,
	GDK_tslash = 956,
	GDK_twofifths = 2739,
	GDK_twosubscript = 16785538,
	GDK_twosuperior = 178,
	GDK_twothirds = 2737,
	GDK_u = 117,
	GDK_uacute = 250,
	GDK_ubelowdot = 16785125,
	GDK_ubreve = 765,
	GDK_ucircumflex = 251,
	GDK_udiaeresis = 252,
	GDK_udoubleacute = 507,
	GDK_ugrave = 249,
	GDK_uhook = 16785127,
	GDK_uhorn = 16777648,
	GDK_uhornacute = 16785129,
	GDK_uhornbelowdot = 16785137,
	GDK_uhorngrave = 16785131,
	GDK_uhornhook = 16785133,
	GDK_uhorntilde = 16785135,
	GDK_umacron = 1022,
	GDK_underbar = 3014,
	GDK_underscore = 95,
	GDK_union = 2269,
	GDK_uogonek = 1017,
	GDK_uparrow = 2300,
	GDK_upcaret = 2985,
	GDK_upleftcorner = 2540,
	GDK_uprightcorner = 2539,
	GDK_upshoe = 3011,
	GDK_upstile = 3027,
	GDK_uptack = 3022,
	GDK_uring = 505,
	GDK_utilde = 1021,
	GDK_v = 118,
	GDK_variation = 2241,
	GDK_vertbar = 2552,
	GDK_vertconnector = 2214,
	GDK_voicedsound = 1246,
	GDK_vt = 2537,
	GDK_w = 119,
	GDK_wacute = 16785027,
	GDK_wcircumflex = 16777589,
	GDK_wdiaeresis = 16785029,
	GDK_wgrave = 16785025,
	GDK_x = 120,
	GDK_xabovedot = 16785035,
	GDK_y = 121,
	GDK_yacute = 253,
	GDK_ybelowdot = 16785141,
	GDK_ycircumflex = 16777591,
	GDK_ydiaeresis = 255,
	GDK_yen = 165,
	GDK_ygrave = 16785139,
	GDK_yhook = 16785143,
	GDK_ytilde = 16785145,
	GDK_z = 122,
	GDK_zabovedot = 447,
	GDK_zacute = 444,
	GDK_zcaron = 446,
	GDK_zerosubscript = 16785536,
	GDK_zerosuperior = 16785520,
	GDK_zstroke = 16777654,
}
alias Keysyms = GdkKeysyms;
